import java.util.*;

public class PathTrace
{
  public static final int MAX_DEPTH = 8; 
  public static FloatColor trace(ArrayList<Object3> scene, Ray r, int depth)
  {
    if (depth == MAX_DEPTH) {
      return FloatColor.BLACK;
    }
    double nearT = Double.POSITIVE_INFINITY;
    Object3 nearObj = null;

    for (Object3 obj : scene) {
      double t = obj.intersect(r);
      if (t == Double.NaN) {
        continue;
      }
      if (t < nearT) {
        nearT = t;
        nearObj = obj;
      }
    }

    if (nearObj == null) {
      return FloatColor.BLACK;
    }

    Material m = nearObj.material;
    FloatColor emittance = m.emittance;

    Vector3 newOrigin = r.valueAt(nearT);
    Vector3 normal = nearObj.normal(newOrigin);
    Vector3 newDir = null;
    boolean transmitted = false;
    
    if (Math.random() > m.transparency && normal.dot(r.direction) < 0) {
      newDir = m.reflect(r.direction, normal);
    } else {
      if (normal.dot(r.direction) < 0) {
        newDir = Refract.refract(r.direction, normal, 1.0, m.ior);
      } else {
        newDir = Refract.refract(r.direction, normal, m.ior, 1.0);
      }
      transmitted = true;
    }

    if (newDir == null) { 
      return emittance;
    }
    
    Ray newRay = new Ray(newOrigin, newDir);
    FloatColor reflected = trace(scene, newRay, depth + 1);
    double cost;
    
    if (!transmitted) {
      cost = newRay.direction.dot(normal);
    } else {
      cost = 1.0;
    }
    
    FloatColor addition = reflected.mult(m.reflectance).mult(cost);
    return emittance.add(addition);
  }
}
