import java.util.*;
import java.awt.*;
import java.applet.*;

public class TraceTest extends Applet
{
  private int width, height;

  public void init() 
  {
     width = getSize().width;
     height = getSize().height;
     setBackground(Color.WHITE);
  }

  public void paint (Graphics g)
  {
    Material lamp = new Material(FloatColor.WHITE.mult(8),
                                 FloatColor.BLACK);
    Material red = new Material(FloatColor.BLACK, FloatColor.RED.mult(0.9));
    Material redMirror = new Material(FloatColor.BLACK, FloatColor.RED.mult(0.9), new SpecularFunction());
    Material green = new Material(FloatColor.BLACK, FloatColor.GREEN.mult(0.9));
    Material greenMirror = new Material(FloatColor.BLACK, FloatColor.GREEN.mult(0.9), new SpecularFunction());
    Material blue = new Material(FloatColor.BLACK, FloatColor.BLUE.mult(0.9));
    Material white = new Material(FloatColor.BLACK, FloatColor.WHITE.mult(0.9));
    Material mirror = new Material(FloatColor.BLACK, FloatColor.WHITE.mult(0.9), new SpecularFunction());
    Material glass = new Material(FloatColor.BLACK, FloatColor.WHITE.mult(0.9), new SpecularFunction(), 1.7, 0.9);

    ArrayList<Object3> scene = new ArrayList<Object3>();

    Sphere mirrorSphere = new Sphere(new Vector3(-2, -1, 3.0), 1.0, mirror);
    Sphere glassSphere =  new Sphere(new Vector3(2, -3, 1), 1, glass);
    Sphere diffuseSphere =  new Sphere(new Vector3(0, -2, 1), 1, white);
    XYRectangle lampRectangle = new XYRectangle (lamp, 5, -1, -1.5, 1.5, -4, -1);
    XYPlane floor = new XYPlane(white, -1, 1);
    XYPlane roof = new XYPlane(white, 5, -1);
    YZPlane left = new YZPlane(red,-3, 1);
    YZPlane right = new YZPlane(green, 3, -1);
    XZPlane front = new XZPlane(white, 1, -1);
    XZPlane back = new XZPlane(white, -12, 1);
    ZCylinder cyl = new ZCylinder(new Vector3(-2, -1, 0), 1, white);
    CappedZCylinder cap = new CappedZCylinder(cyl, -1, 2);

    scene.add(glassSphere);
    scene.add(mirrorSphere);
    scene.add(diffuseSphere);
    scene.add(cap);
    scene.add(lampRectangle);
    scene.add(floor);
    scene.add(roof);
    scene.add(left);
    scene.add(right);
    scene.add(front);
    scene.add(back);
    FloatColor[][] screen = new FloatColor[301][301];
    for (int i = 0; i < 301; i++) {
      for (int j = 0; j < 301; j++) {
        screen[i][j] = FloatColor.BLACK;
      }
    }
    Vector3 eye = new Vector3(0, -15, 2);
    for (int i = 1;;i++) {
      g.setColor(Color.WHITE);
      g.fillRect(0, 301, 300, 10);
      for (double x = -150; x <= 150; x++) {
        for (double y = -150; y <= 150; y++) {
          double vx = (x + Math.random()) / 50;
          double vy = (y + Math.random()) / 50;
          Vector3 point = new Vector3(vx, -5, vy + 2);    
          Ray r = new Ray(point, point.sub(eye));
          FloatColor c = PathTrace.trace(scene, r, 0);
          screen[(int) (x + 150)][(int) (y + 150)] = screen[(int) (x + 150)][(int) (y + 150)].add(c);
        }
        g.setColor(Color.RED);
        g.fillRect((int) x + 150, 301, 1, 10);
      }
      for (int row = 0; row < 301; row++) {
        for (int col = 0; col < 301; col++) {
          Color c = screen[row][col].mult(1.0 / ((double) i)).toColor();
          g.setColor(c);
          g.fillRect(row, 301 - col, 1, 1);
        }
      }
    }
  }
}
