public class ZCylinder extends Object3
{
  public Vector3 center;
  public double radius;

  public ZCylinder(Vector3 v, double r, Material m)
  {
    super(m);
    center = v;
    radius = r;
  }

  public double intersect(Ray r)
  {
    double x = center.x;
    double y = center.y;
    double dir_x = r.direction.x;
    double dir_y = r.direction.y;
    double o_x = r.origin.x;
    double o_y = r.origin.y;

    double u = o_x - x;
    double v = o_y - y;
    
    double a = dir_x * dir_x + dir_y * dir_y;
    double b = 2 * (dir_x * u + dir_y * v);  
    double c = u * u + v * v - radius * radius;

    double disc = b * b - 4 * a * c;
    
    if (disc < 0) return Double.NaN;

    double term1 = -b / (2 * a);
    double term2 = Math.sqrt(disc) / (2 * a);

    double t1 = term1 - term2;
    if (t1 > 0) return t1;
    
    double t2 = term1 + term2;
    if (t2 > 0) return t2;

    return Double.NaN;
  }

  public Vector3 normal(Vector3 p)
  {
    return new Vector3 (p.x - center.x, p.y - center.y, p.z);
  }
}
